import sys
from os.path import dirname
from kerykeion.main import KrInstance
from kerykeion.output_exemple import output
from kerykeion.utilities import kr_settings as settings
from kerykeion.utilities.general import print_settings_path
from kerykeion.utilities.charts import MakeSvgInstance as MakeSvgInstance
from kerykeion.utilities.discepolo import DiscepoloNumber


sys.path.append(dirname(__file__))
